
# q-expansion of newform 324.8.e.h, downloaded from the LMFDB on 11 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 324
weight = 8
poly_data = [4225, 0, -65, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, 0, 1, 0], 65], [[0, 2340, 0, 36], 65], [[0, 4680, 0, -36], 65]]

hecke_ring_character_values = [[163, [1, 0, 0, 0]], [245, [0, -1, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 0], [0, 0, -1, 0], [-1553, 1553, 0, 0], [0, 0, 13, -13], [0, 799, 0, 0], [0, 0, 0, 23], [-35533, 0, 0, 0], [0, 0, 211, 0], [0, 0, -296, 296], [0, -77828, 0, 0], [237383, 0, 0, 0], [0, 0, 752, 0], [26200, -26200, 0, 0], [0, 0, 777, -777], [0, 0, 0, -3458], [0, 0, -389, 0], [-423059, 423059, 0, 0], [0, -1950023, 0, 0], [0, 0, 0, -3214], [-3470239, 0, 0, 0], [-4678583, 4678583, 0, 0], [0, 0, 3810, -3810], [0, 0, 0, -5197], [8300083, -8300083, 0, 0], [0, 0, -11944, 11944], [0, 5271091, 0, 0], [0, 0, 0, 8743], [-19907206, 0, 0, 0], [0, 0, -2763, 0], [26899580, 0, 0, 0], [0, 0, 35948, 0], [0, 0, -99227, 99227], [0, -30112517, 0, 0], [0, 0, -26762, 0], [-26506931, 26506931, 0, 0], [0, -7357910, 0, 0], [-31673251, 0, 0, 0], [0, 0, -120825, 0], [0, 0, 66768, -66768], [0, 0, 0, 38038], [104429501, 0, 0, 0], [0, 0, 174815, -174815], [0, -16959719, 0, 0], [0, 0, 0, 236861], [28743893, 0, 0, 0], [0, -242017409, 0, 0], [258919300, -258919300, 0, 0], [0, 0, -537270, 537270], [0, -155957666, 0, 0], [0, 0, 0, 452188], [0, 0, 268758, 0], [-107999249, 107999249, 0, 0], [0, 0, 0, 503692], [0, 0, 655854, 0], [0, 0, -557050, 557050], [0, 0, 0, -931937], [-224404021, 0, 0, 0], [431506270, -431506270, 0, 0], [0, 0, -689168, 689168], [0, 72517360, 0, 0], [0, 0, 1353493, 0], [639456320, 0, 0, 0], [0, 0, -1005583, 0], [-289941899, 289941899, 0, 0], [0, 0, -707518, 707518], [325389091, -325389091, 0, 0], [0, 512510443, 0, 0], [0, 0, 843054, 0], [-883411007, 883411007, 0, 0], [0, 0, 2817874, -2817874], [0, 0, 0, 906567], [-906675557, 906675557, 0, 0], [0, -1556922431, 0, 0], [508826087, 0, 0, 0], [0, 0, 2677736, 0], [0, 0, -3263767, 3263767], [-1536486010, 0, 0, 0], [0, 0, -3773734, 0], [0, 2022108343, 0, 0], [0, 0, 1812567, 0], [822229519, -822229519, 0, 0], [0, 0, 0, -5762179], [2016605810, 0, 0, 0], [30170116, -30170116, 0, 0], [0, 0, 1745994, -1745994], [0, 0, 0, -4733271], [313302250, -313302250, 0, 0], [0, 0, -6899585, 6899585], [0, -2322490241, 0, 0], [0, 0, 0, -878783], [0, 0, 8455854, -8455854], [-2846015047, 0, 0, 0], [0, 0, 1034329, 0], [0, 5088816256, 0, 0], [0, 0, 0, 3094797], [0, 0, -4450573, 0], [0, 0, 0, 11843849], [-1640617489, 0, 0, 0], [-4949703271, 0, 0, 0], [2139243403, -2139243403, 0, 0], [0, 0, 0, 5298599], [0, 0, 1021368, 0], [0, 0, -7828729, 7828729], [0, -6118740659, 0, 0], [1591249787, 0, 0, 0], [0, 0, -12963877, 12963877], [0, 0, 0, 8905634], [0, 0, 8785768, 0], [-152504378, 152504378, 0, 0], [0, -8237054153, 0, 0], [-5578211701, 0, 0, 0], [0, 0, -389761, 0], [-2366536163, 2366536163, 0, 0], [12011503499, 0, 0, 0], [0, 0, 15092242, -15092242], [0, -10040473400, 0, 0], [0, 0, 0, 20053416], [0, 0, -13003646, 0], [0, 0, -27012238, 27012238], [0, -7036222331, 0, 0], [-3422586011, 3422586011, 0, 0], [0, 0, -3342595, 3342595], [0, 0, 0, -4027812], [3741736312, -3741736312, 0, 0], [0, 0, 0, 8106103], [-12652348937, 12652348937, 0, 0], [0, 0, 0, -2724594], [-16305213140, 16305213140, 0, 0], [0, 17889720790, 0, 0], [3230012576, 0, 0, 0], [0, 0, -24589701, 0], [0, 8705864689, 0, 0], [-9367843807, 0, 0, 0], [0, 0, 15574809, 0], [0, -6421984463, 0, 0], [0, 0, 0, 37055970], [0, -3434291723, 0, 0], [0, 0, 10012932, 0], [0, 0, 0, 6668798], [-6168814792, 0, 0, 0], [0, 0, -31807003, 31807003], [0, -22209117419, 0, 0], [0, 0, 0, -27574639], [-880596523, 0, 0, 0], [0, 0, -2101410, 2101410], [-12986725631, 12986725631, 0, 0], [0, 0, 25342302, -25342302], [0, 1148006167, 0, 0], [0, 0, 0, -60983455], [0, 12519185857, 0, 0], [0, 0, 0, 33177721], [-7426691659, 0, 0, 0], [0, 0, -32607310, 0], [-21089573303, 21089573303, 0, 0], [0, 0, -52198428, 52198428], [-14544735244, 0, 0, 0], [0, 0, -47220426, 47220426], [-4031040997, 0, 0, 0], [0, 0, 70720885, 0], [0, 0, -8395197, 8395197], [0, 0, 0, 66079407], [0, -30585082583, 0, 0], [0, 0, 0, 11556567], [0, 0, -12994226, 0], [0, 0, -72204769, 72204769], [-39080139211, 0, 0, 0], [23546282770, -23546282770, 0, 0]]
